#!/bin/bash

pub_port=5733
stb1_port=5734
stb2_port=5735
stb3_port=5736

echo '=========='
echo '=Clean up='
echo '=========='

./pg_ctl stop -D ../../primary
rm -rf  ../../primary
rm -rf  ../../primary.log

./pg_ctl stop -D ../../standbydb1
rm -rf  ../../standbydb1
rm -rf  ../../standbydb1.log

./pg_ctl stop -D ../../standbydb2
rm -rf  ../../standbydb2
rm -rf  ../../standbydb2.log

./pg_ctl stop -D ../../standbydb3
rm -rf  ../../standbydb3
rm -rf  ../../standbydb3.log

echo '=================================================================================================='
echo '=Set up primary server='
echo '=================================================================================================='

./initdb -D ../../primary

cat << EOF >> ../../primary/postgresql.conf
port = $pub_port
wal_level = logical
standby_slot_names = 'standby_1,standby_2,standby_3' 

max_wal_senders=100
max_replication_slots=1000 
log_replication_commands = 'on' 
checkpoint_timeout = 1d 
shared_buffers = 6GB 
wal_buffers = '1GB'
synchronous_commit = on 
checkpoint_timeout = 1d 
max_wal_size = 20GB
min_wal_size = 10GB

autovacuum = off 
EOF

cat << EOF >> ../../primary/pg_hba.conf
host all,replication all 0.0.0.0/0 trust
EOF

./pg_ctl -D ../../primary -w -l ../../primary.log start

./psql -p $pub_port -d postgres -c "SELECT pg_create_physical_replication_slot('standby_1');"
./psql -p $pub_port -d postgres -c "SELECT pg_create_physical_replication_slot('standby_2');"
./psql -p $pub_port -d postgres -c "SELECT pg_create_physical_replication_slot('standby_3');"
./psql -p $pub_port -d postgres -c "CREATE ROLE replication WITH REPLICATION PASSWORD 'password' LOGIN;"
./pg_ctl -D ../../primary -w -l ../../primary.log restart

echo '===================================================================================================='
echo '=Set up standby servers'
echo '===================================================================================================='

./pg_basebackup -h 127.0.0.1 -D ../../standbydb1  -P -U replication -X s  -p $pub_port

cat << EOF >> ../../standbydb1/postgresql.conf
port = $stb1_port
primary_slot_name = 'standby_1'
hot_standby_feedback = on
#sync_replication_slots = true
standby_slot_names = ''

primary_conninfo = 'port=$pub_port application_name=secondary_1 dbname=postgres user=replication'
EOF

sed -i 's/hosts=disable/hosts=disable dbname=postgres/g' ../../standbydb1/postgresql.auto.conf

cat << EOF >> ../../standbydb1/standby.signal
EOF

./pg_ctl -D ../../standbydb1 -w -l ../../standbydb1.log start

##########################################################################################################

./pg_basebackup -h 127.0.0.1 -D ../../standbydb2 -P -U replication -X s  -p $pub_port

cat << EOF >> ../../standbydb2/standby.signal
EOF

cat << EOF  >> ../../standbydb2/postgresql.conf
port = $stb2_port
primary_slot_name = 'standby_2'
hot_standby_feedback = on
#sync_replication_slots = true
standby_slot_names = ''
primary_conninfo = 'port=$pub_port application_name=secondary_2 dbname=postgres user=replication'
EOF

sed -i 's/hosts=disable/hosts=disable dbname=postgres/g' ../../standbydb2/postgresql.auto.conf

./pg_ctl -D ../../standbydb2 -w -l ../../standbydb2.log start
##########################################################################################################

./pg_basebackup -h 127.0.0.1 -D ../../standbydb3 -P -U replication -X s  -p $pub_port


cat << EOF >> ../../standbydb3/standby.signal
EOF

cat << EOF >> ../../standbydb3/postgresql.conf
port = $stb3_port
primary_slot_name = 'standby_3'
hot_standby_feedback = on
#sync_replication_slots = true
standby_slot_names = ''
primary_conninfo = 'port=$pub_port application_name=secondary_3 dbname=postgres user=replication'
EOF

sed -i 's/hosts=disable/hosts=disable dbname=postgres/g' ../../standbydb3/postgresql.auto.conf

./pg_ctl -D ../../standbydb3 -w -l ../../standbydb3.log start

pgbench -p $pub_port postgres -qis 2


./pg_ctl -D ../../standbydb1 -w -l ../../standbydb1.log  stop
./pg_ctl -D ../../standbydb2 -w -l ../../standbydb2.log  stop
./pg_ctl -D ../../standbydb3 -w -l ../../standbydb3.log  stop

psql -d postgres -p $pub_port -f ./slots.sql

./pg_ctl -D ../../standbydb1 -w -l ../../standbydb1.log  start
./pg_ctl -D ../../standbydb2 -w -l ../../standbydb2.log  start
./pg_ctl -D ../../standbydb3 -w -l ../../standbydb3.log  start

##########################################################################################################
# pgbench
##########################################################################################################

#
# pgbench --aggregate-interval=10 --time=20 --client=10 --log --rate=1000 --latency-limit=10 --failures-detailed --max-tries=10 -p 5733 postgres
#
